#!/bin/sh
changequote([[,]])dnl
changecom([[##]])dnl
#
# Startup script for the DNS caching server under ALTLinux
#
# chkconfig: - 20 55
# description: This script starts your DNS caching server
# processname: dnsmasq
# config: /etc/dnsmasq.conf
# pidfile: /var/run/dnsmasq.pid

# Do not load RH compatibility interface.
WITHOUT_RC_COMPAT=1

# Source function library.
. /etc/init.d/functions

if [ "$1" = "--trace-script" ]; then
    [[shift]]
    set -x
fi

# Source networking configuration and check that networking is up.
SourceIfNotEmpty /etc/sysconfig/network
is_yes "${NETWORKING}" || exit 0

dnsmasq=/usr/sbin/dnsmasq
[ -x $dnsmasq ] || exit 0

# Assign default values, then load configuration file.
OPTIONS=""
ALL=255.255.255.255
DOMAIN_SUFFIX=$(dnsdomainname 2>/dev/null || hostname --domain 2>/dev/null)
ifdef([[with_resolvconf]],,[[dnl
PPP_RESOLV_CONF=/etc/ppp/resolv.conf
SYNC_RESOLV_CONF=yes
]])dnl
SourceIfExists /etc/sysconfig/dnsmasq

# Build program command line
[ -n "${MAILHOSTNAME}"    ] && OPTIONS="$OPTIONS -m $MAILHOSTNAME"
ifdef([[with_resolvconf]],,[[dnl
[ -n "${PPP_RESOLV_CONF}" ] && OPTIONS="$OPTIONS -r $PPP_RESOLV_CONF"
]])dnl
[ -n "${RESOLV_CONF}"     ] && OPTIONS="$OPTIONS -r $RESOLV_CONF"
[ -n "${DOMAIN_SUFFIX}"   ] && OPTIONS="$OPTIONS -s $DOMAIN_SUFFIX"
[ -n "${DHCP_LEASE}"      ] && OPTIONS="$OPTIONS -l $DHCP_LEASE"

# Internal variables
PIDFILE=/var/run/dnsmasq.pid
LOCKFILE=/var/lock/subsys/dnsmasq
RETVAL=0

# External commands
CP=/bin/cp
MV=/bin/mv
RM=/bin/rm
TOUCH=/bin/touch
EGREP=/bin/egrep
ifdef([[with_resolvconf]],[[dnl
RESOLVCONF=/sbin/resolvconf
]])dnl

#---------------------------------------------------------------
#
#	/etc/resolv.conf
#

ifdef([[with_resolvconf]],[[dnl
start_resolvconf()
{
    if [ -n "$RESOLV_CONF" -a -n "$CONF_RESOLVCONF" ]; then
        "$TOUCH" "$RESOLV_CONF" "$CONF_RESOLVCONF"
        local msg="Start resolvconf:"
        echo -n "$msg"
        echo 'nameserver 127.0.0.1' | /sbin/resolvconf -a lo.dnsmasq &&
            success "$msg" || failure "$msg"
        echo
    fi
}

stop_resolvconf()
{
    if [ -n "$RESOLV_CONF" -a -n "$CONF_RESOLVCONF" ]; then
        # Don't remove files if another dnsmasq running under NetworkManager
        if ! ls -1 /var/run/ /var/lib/run/ 2>/dev/null | grep -qs '^nm-dnsmasq'; then
            "$RM" -f "$RESOLV_CONF" "$CONF_RESOLVCONF"
        fi
        action "Stop resolvconf:" "$RESOLVCONF" -fd lo.dnsmasq
    fi
}
]],[[dnl
notify_resolver() {
#   ExecIfExecutable /sbin/update_chrooted conf
    ExecIfExecutable /etc/chroot.d/resolv.conf
}

function check_resolv_conf() {
    case "$SYNC_RESOLV_CONF" in
	YES|Yes|yes|Y|y|TRUE|True|true|ON|On|on|1 ) ;;
	* ) return 1
    esac
    [ -z "$RESOLV_CONF" ] && return 1
    [ "$RESOLV_CONF" = "/etc/resolv.conf" ] && return 1
    return 0
}

function do_resolv_conf()
{
    check_resolv_conf || return 1

    if [ /etc/resolv.conf -nt "$RESOLV_CONF" ]; then
	echo "Update/create $RESOLV_CONF"
#	$EGREP -vi '(^ *nameserver 127.0.0.1)|(# ppp temp entry)' /etc/resolv.conf > "$RESOLV_CONF"
	cp -af /etc/resolv.conf "$RESOLV_CONF"
    fi

    # Is /etc/resolv.conf already valid?
    local     n_ppp=`$EGREP -c '# ppp temp entry' /etc/resolv.conf`
    local      n_ns=`$EGREP -c '^ *nameserver' /etc/resolv.conf`
    local n_localns=`$EGREP -c '^ *nameserver *127.0.0.1' /etc/resolv.conf`

    if [ "$n_ppp" != "0" -o "$n_ns" != "$n_localns" ]; then
	echo "Validate /etc/resolv.conf"
	$EGREP -vi '(^ *nameserver *)|(# ppp temp entry)' /etc/resolv.conf > /etc/resolv.conf.tmpnew
	echo -e "\nnameserver 127.0.0.1" >> /etc/resolv.conf.tmpnew
	$MV -f /etc/resolv.conf.tmpnew /etc/resolv.conf
	$TOUCH -r "$RESOLV_CONF" /etc/resolv.conf
	notify_resolver
    fi

    return 0
}

function undo_resolv_conf()
{
    check_resolv_conf || return 1
    [ -r "$RESOLV_CONF" ] || return 1

    if [ /etc/resolv.conf -nt "$RESOLV_CONF" ]; then
	echo "WARNING: /etc/resolv.conf is newer than $RESOLV_CONF"
	echo "         It will be saved to /etc/resolv.conf.dnsmasqnew"
	$MV -f /etc/resolv.conf /etc/resolv.conf.dnsmasqnew
    fi

    echo "Restore /etc/resolv.conf"
    $CP -af "$RESOLV_CONF" /etc/resolv.conf
    notify_resolver

    return 0
}
]])dnl

#---------------------------------------------------------------
#
#	Broadcast routing
#

function print_all_ifaces()
{
    local linenum=0
    netstat -vai | while read iface moredata; do
	[ $[++linenum] -le 2 ] && continue
	case "$iface" in
	    lo )  ;;  # skip loopback
    #       *:* ) ;;  # skip aliases?
	    * ) echo $iface ;;
	esac
    done
}

function print_listening_ifaces()
{
    local line words dev
    netstat -ltnp | egrep '[0-9]*/dnsmasq' | egrep ':53 ' | while read line; do
	words=($line)
	dev="${words[3]%:*}"
	if [ "$dev" = "0.0.0.0" ]; then
	    print_all_ifaces
	    break
	fi
	echo "$dev"
    done | uniq
}

function do_route() {
    [ -n "$ALL_DEV" ] || return
    local msg='Broadcast device ALL_DEV is defined, but destaddr ALL is empty!'
    if [ -z "$ALL" ]; then
	echo -n $msg
	failure "$msg"; echo
	return 1
    fi
    local cmd=$1
    [[shift]]
    echo -n $*
    start_daemon --no-announce -- \
	/sbin/ip route $cmd $ALL dev $ALL_DEV
}

function cleanup_routing()
{
    [ -z "$ALL" ] && return
    /sbin/ip route list | $EGREP "^$ALL dev " | while read dst dev_kword iface tail; do
	/sbin/ip route del $dst dev $iface $tail
    done
}

#---------------------------------------------------------------
#
#	Main routines
#

function start()
{
ifdef([[with_resolvconf]],[[dnl
    [ -z "$1" ] && start_resolvconf
]],[[dnl
    do_resolv_conf
]])dnl
    start_daemon --pidfile "$PIDFILE" --lockfile "$LOCKFILE" --expect-user nobody -- $dnsmasq $OPTIONS
    RETVAL=$?
    if [ $RETVAL -eq 0 ]; then
        do_route add "Adding local broadcast host route:"
    else
ifdef([[with_resolvconf]],[[dnl
        stop_resolvconf
]],[[dnl
        undo_resolv_conf
]])dnl
    fi
    return $RETVAL
}

function start_debug()
{
ifdef([[with_resolvconf]],[[dnl
    "$RESOLVCONF" -u
]],[[dnl
    do_resolv_conf
]])dnl
    # don't adds/removes routing!
    $dnsmasq -d -q $OPTIONS
ifdef([[with_resolvconf]],,[[dnl
    undo_resolv_conf
]])dnl
}

function stop()
{
ifdef([[with_resolvconf]],,[[dnl
    undo_resolv_conf
]])dnl
    do_route del "Removing host route defined at startup:"
    stop_daemon --pidfile "$PIDFILE" --lockfile "$LOCKFILE" --expect-user nobody -- $dnsmasq
    RETVAL=$?
ifdef([[with_resolvconf]],[[dnl
    [ -z "$1" ] && stop_resolvconf
]])dnl
    return $RETVAL
}

function my_status() {
    status --pidfile "$PIDFILE" --expect-user nobody -- $dnsmasq
    RETVAL=$?
    if [ $RETVAL = 0 ]; then
	echo "Listening interfaces:"
	print_listening_ifaces
    fi
    return $RETVAL
}

function restart() {
ifdef([[with_resolvconf]],[[dnl
    # Do not call resolvconf during restart to avoid loop
    # if restarting from resolvconf itself
    stop restart
    start restart
]],[[dnl
    stop
    start
]])dnl
    RETVAL=$?
    return $RETVAL
}

function reload() {
    msg_reloading dnsmasq
    stop_daemon --pidfile "$PIDFILE" --expect-user nobody -HUP -- $dnsmasq
    RETVAL=$?
    return $RETVAL
}

function clean() {
    stop
    $RM -f $LOCKFILE $PID_FILE
ifdef([[with_resolvconf]],,[[dnl
    check_resolv_conf && $RM -f "$RESOLV_CONF"
]])dnl
    cleanup_routing
    RETVAL=0
}

case "$1" in
    start)   start ;;
    stop)    stop ;;
    status)  my_status ;;
    restart) restart ;;
    reload)  reload ;;
    clean)   clean ;;
    startdebug)  start_debug ;;
    condrestart) [ -e "$LOCKFILE" ] && restart ;;
    condreload)  [ -e "$LOCKFILE" ] && reload ;;
    condstop)    [ -e "$LOCKFILE" ] && stop ;;
    *)
        echo "Usage: ${0##*/} {start|stop|restart|reload|condrestart|condreload|condstop|clean|status}"
        exit 1
esac

exit $RETVAL

## EOF ##
